% Timoshenko Beam - based on BJ Stone paper P I Mech Eng: PtC J Mech Eng Sc Vol 206, 1992. pp87-94.
%-------------------------------------------------------------------------------------------------------------
function recept_timoshenko=beam_recept_timoshenko(w,E,rho,b,d,len,shape,G,nu)
% recept_euler = 2x2 tip receptance of the whole beam at freq w.
%Here the number of sections is set to be the number of depth values
sections=length(d);
%the number of values in width and length should be the same as depth, otherwise, cannot calculate in different values
if(length(b)~=sections)||(length(len)~=sections)
error('Vectors for breadth, depth and length must be same size.');
end;
%if x_section is set to be retangular ot circular, the flexural rigidity and cross sectional area can be calculated, and the type of x_section is passed from main function
if strcmp(shape,'rectangular')
I=b.*d.^3./12;
EI=E.*I; % Flexural rigidity for each section
area=b.*d; % Cross section area of each section
% Calculates Hutchinson's factor for each element
%lambda=shear_factor_timoshenko(nu,b,d);
lambda=shear_factor_timoshenko(nu,b,d);
elseif strcmp(shape,'circular')
I=pi.*d.^4./64;
EI=E.*I;
area=pi.*d.*d./4;
%Shear shape factor. Ref.S.S. Rao, Mechanical Vibrations, 4th Ed,p624.
lambda=9/10;
else
error('Unknown cross-sectional shape');
end;
for k=1:sections
aplus(k)=1+E./(lambda(k).*G);
aminus(k)=1-E./(lambda(k).*G);
end;
%there are two sets of equations to calculate tip receptances, and it depends on whetehr w*w is larger or smaller than
%lambda(k).*G.*area(k)./(rho.*I(k))
signs_ref=[-1 1 1 1 -1 -1 -1 -1 -1 1 1 -1 -1 1];
% Need to do this since each section may have different lambda(k).*G.*area./(rho.*I)
for k=1:sections
if (w.*w < lambda(k).*G.*area(k)./(rho.*I(k)))
signs=signs_ref;
else
signs=-signs_ref;
end;
m(k)=rho*I(k)*w*w;
n(k)=sqrt(m(k)*(m(k).*aminus(k)^2+4*E*area(k)));
eta(k)=sqrt(((signs(1)*m(k)*aplus(k)) + signs(2)*n(k))/(2*EI(k)));
xi(k)=sqrt(((m(k)*aplus(k)) + n(k))/(2*EI(k)));
s(k)=rho*w*w/(lambda(k)*G*xi(k))-xi(k);
r(k)=rho*w*w/(lambda(k)*G*eta(k))+signs(3)*eta(k);
e(k)=r(k)*eta(k)^2/(s(k)*xi(k)^2);
xi_len(k)=xi(k)*len(k);
eta_len(k)=eta(k)*len(k);
c(k)=cos(xi_len(k)); ch(k)=cosh(eta_len(k));
si(k)=sin(xi_len(k)); sh(k)=sinh(eta_len(k));
f1(k)=si(k)*sh(k);
f2(k)=c(k)*ch(k);
f3(k)=f2(k)-1;
f4(k)=si(k)*ch(k);
f5(k)=c(k)*sh(k);
f6(k)=c(k)-ch(k);
recip_delta(k)=1/(rho*w*w*area(k)*(2-2*f2(k)+signs(4)*(e(k)+signs(5)/e(k))*f1(k)));
end;
% tip receptance of first section
if (w*w < lambda(k)*G*area(1)/(rho*I(1)))
signs=signs_ref;
else
signs=-signs_ref;
end;
beta=recip_delta(1)*[signs(6)*f5(1)*(r(1)*eta(1)^2/(s(1)*xi(1))-eta(1))+f4(1)*(s(1)*xi(1)^2/(r(1).*eta(1))-xi(1)),...
signs(7)*f3(1)*(s(1)*xi(1)+r(1)*eta(1))+signs(8)*f1(1)*(r(1)*eta(1)^2/xi(1)+signs(9)*s(1).*xi(1).^2./eta(1));...
signs(7)*f3(1)*(s(1)*xi(1)+r(1)*eta(1))+signs(8)*f1(1)*(r(1)*eta(1)^2/xi(1)+signs(9)*s(1).*xi(1).^2./eta(1)),...
f4(1)*r(1)*eta(1)*(s(1)-r(1)*eta(1)/xi(1))+f5(1)*s(1)*xi(1)*(r(1)-s(1)*xi(1)/eta(1))];
% add all of the remaining sections in section C
for k=2:sections
if (w*w < lambda(k).*G.*area(k)/(rho.*I(k)))
signs=signs_ref;
else
signs=-signs_ref;
end;
gamma1=recip_delta(k).*[signs(6).*f5(k).*(r(k).*eta(k).^2./(s(k).*xi(k))-eta(k))+f4(k).*(s(k).*xi(k).^2./(r(k).*eta(k))-xi(k)),...
signs(10).*f3(k).*(s(k).*xi(k)+r(k).*eta(k))+signs(11).*f1(k).*(r(k).*eta(k).^2./xi(k)+signs(9).*s(k).*xi(k).^2./eta(k));...
signs(10).*f3(k).*(s(k).*xi(k)+r(k).*eta(k))+signs(11).*f1(k).*(r(k).*eta(k).^2./xi(k)+signs(9).*s(k).*xi(k).^2./eta(k)),...
f4(k).*r(k).*eta(k).*(s(k)-r(k).*eta(k)./xi(k))+f5(k).*s(k).*xi(k).*(r(k)-s(k).*xi(k)./eta(k))];
gamma2=recip_delta(k).*[signs(12).*sh(k).*(r(k).*eta(k).^2./(s(k).*xi(k))-eta(k))+si(k).*(s(k).*xi(k).^2./(r(k).*eta(k))-xi(k)),...
signs(13).*f6(k).*(r(k).*eta(k)-s(k).*xi(k));...
signs(14).*f6(k).*(r(k).*eta(k)-s(k).*xi(k)),...
sh(k).*(r(k).*s(k).*xi(k)-s(k).^2.*xi(k).^2./eta(k))+si(k).*(r(k).*s(k).*eta(k)-r(k).^2.*eta(k).^2./xi(k))];
gamma3=gamma1.*[1 -1; -1 1];
alpha=gamma3-gamma2'*((gamma1+beta)\gamma2); % build the structure
beta=alpha; % new tip receptance ready for next section to be added
end;
recept_timoshenko=alpha; % This is a 2x2 matrix of tip receptances.